@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.9"
+__version__ = "0.1.12"
 
 
 """
@@ -31,8 +31,12 @@ def test_prokka():
     :return:
     """
     s = Sample("Synechococcus", files={"assembly": synechococcus_genome})
-    _ = prokka(s)
-    pass
+    prokka_program = prokka(s)
+    prokka_params = list(prokka_program.serializer()["params"].keys())
+
+    expected_params = ["--prefix", "--outdir", "--cpus", ""]
+
+    assert prokka_params == expected_params
 
 
 def test_kaiju():
@@ -40,8 +44,13 @@ def test_kaiju():
     Testing the 'kaiju' program.
     :return:
     """
-    s = Sample("Synechococcus", files={"R1": synechococcus_genome, "R2": ""})
-    _ = kaiju(s)
+    s = Sample("Synechococcus", files={"R1": synechococcus_genome, "R2": "r2.fastq"})
+    kaiju_program = kaiju(s)
+    kaiju_params = list(kaiju_program.serializer()["params"].keys())
+
+    expected_params = ["-t", "-i", "-j", "-f", "-z", "-o"]
+
+    assert kaiju_params == expected_params
 
 
 def test_kaiju2table():
@@ -50,4 +59,9 @@ def test_kaiju2table():
     :return:
     """
     s = Sample("Synechococcus", files={"kaiju_output": synechococcus_genome})
-    _ = kaiju2table(s)
+    kaiju2tab_program = kaiju2table(s)
+    kaiju2tab_params = list(kaiju2tab_program.serializer()["params"].keys())
+
+    expected_params = ["-o", "-t", "-n", "-r"]
+
+    assert kaiju2tab_params == expected_params
